<?php
/**
 * Adds gg_Split_Categories_Widget widget.
 */
class gg_Split_Categories_Widget extends WP_Widget {

	/**
	 * Register widget with WordPress.
	 */
	function __construct() {
		parent::__construct(
			'gg_split_categories_widget', // Base ID
			__('Split Categories Widget', 'okthemes'), // Name
			array( 'description' => __( 'Display categories on multiple columns', 'okthemes' ), 'classname' => 'split-categories', ) // Args
		);
	}

	/**
	 * Front-end display of widget.
	 */
	public function widget( $args, $instance ) {
		$title = apply_filters( 'widget_title', $instance['title'] );

		$number = ! empty( $instance['number'] ) ? $instance['number'] : '10';
		$split_number = ! empty( $instance['split_number'] ) ? $instance['split_number'] : '2';

		echo $args['before_widget'];

		// Grab the categories - top level only (depth=1)
		 $get_cats = wp_list_categories( 'echo=0&title_li=&depth=1&show_count=1&number='.$number.'' );
		// Split into array items
		 $cat_array = explode('</li>',$get_cats);
		// Amount of categories (count of items in array)
		 $results_total = count($cat_array);
		// How many categories to show per list (round up total divided by 5)
		 $cats_per_list = ceil($results_total / $split_number);
		// Counter number for tagging onto each list
		 $list_number = 1;
		// Set the category result counter to zero
		 $result_number = 0;
		 // Set the class to split
		 $split_number_class = 'col-xs-12';

		 switch ($split_number) {
		    case 1:
		        $split_number_class = 'col-xs-12';
		        break;
		    case 2:
		        $split_number_class = 'col-xs-6';
		        break;
		    case 3:
		        $split_number_class = 'col-xs-4';
		        break;
		    case 4:
		        $split_number_class = 'col-xs-3';
		        break;        
		}

		if ( ! empty( $title ) )
			echo $args['before_title'] . $title . $args['after_title'];
		?>	

		<ul class="split_categories <?php echo $split_number_class; ?>" id="cat-col-<?php echo $list_number; ?>">
 
			    <?php
			        foreach($cat_array as $category) {
			        $result_number++;
			 
			        if($result_number % $cats_per_list == 0) {
			        $list_number++;
			        echo $category.'</li>
			        </ul>
			 
			        <ul class="split_categories '.$split_number_class.'" id="cat-col-'.$list_number.'">';
			 
			    }
			 
			    else {
			     
			        echo $category.'</li>';
			             
			        }
			 
			    } ?>
			             
			</ul>
		<div class="clearfix"></div>

		<?php
		echo $args['after_widget'];
	}
	

	/**
	 * Back-end widget form.
	 */
	public function form( $instance ) {
		
		$title = isset( $instance['title'] ) ? $instance['title'] : __( 'Contact us', 'okthemes' );
		$number = isset( $instance['number'] ) ? $instance['number'] : '10';
		$split_number = isset( $instance['split_number'] ) ? $instance['split_number'] : '2';

		?>
		<!-- Widget Title: Text Input -->
		<p>
			<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e('Title:', 'okthemes'); ?></label>
			<input type="text" id="<?php echo $this->get_field_id( 'title' ); ?>" class="widefat" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php echo $instance['title']; ?>" />
		</p>
		<p>
		  <label for="<?php echo $this->get_field_id('number'); ?>"><?php _e('Number of categories to display:', 'okthemes'); ?></label>
		  <input type="text" size="2" id="<?php echo $this->get_field_id('number'); ?>" name="<?php echo $this->get_field_name('number'); ?>" type="text" value="<?php echo esc_attr( $number ); ?>" />
		</p>
		<p>
		  <label for="<?php echo $this->get_field_id('split_number'); ?>"><?php _e('Number of columns to split:', 'okthemes'); ?></label>
		  <input type="text" size="2" id="<?php echo $this->get_field_id('split_number'); ?>" name="<?php echo $this->get_field_name('split_number'); ?>" type="text" value="<?php echo esc_attr( $split_number ); ?>" />
		  <br>
		  <small><?php _e('Maximum number of columns is 4', 'okthemes'); ?></small>
		</p>
		<?php 
	}


	/**
	 * Sanitize widget form values as they are saved.
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = array();

		$instance['title'] = ( ! empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';
		$instance['number'] = ( ! empty( $new_instance['number'] ) ) ? strip_tags( $new_instance['number'] ) : '';
		$instance['split_number'] = ( ! empty( $new_instance['split_number'] ) ) ? strip_tags( $new_instance['split_number'] ) : '';
		
		return $instance;
	}


} // class gg_Split_Categories_Widget

// register gg_Split_Categories_Widget 
function register_gg_split_categories_widget() {
    register_widget( 'gg_Split_Categories_Widget' );
}
add_action( 'widgets_init', 'register_gg_split_categories_widget' );